from typing import List, Optional

import attr

from secoda_common.models.user import User
from marshmallow_annotations.ext.attrs import AttrsSchema


@attr.s(auto_attribs=True, kw_only=True)
class Reader:
    user: User
    read_count: int


class ReaderSchema(AttrsSchema):
    class Meta:
        target = Reader
        register_as_scheme = True


@attr.s(auto_attribs=True, kw_only=True)
class Tag:
    tag_type: str
    tag_name: str


class TagSchema(AttrsSchema):
    class Meta:
        target = Tag
        register_as_scheme = True


@attr.s(auto_attribs=True, kw_only=True)
class Badge:
    badge_name: str = attr.ib()
    category: str = attr.ib()


class BadgeSchema(AttrsSchema):
    class Meta:
        target = Badge
        register_as_scheme = True


@attr.s(auto_attribs=True, kw_only=True)
class Watermark:
    watermark_type: Optional[str] = None
    partition_key: Optional[str] = None
    partition_value: Optional[str] = None
    create_time: Optional[str] = None


class WatermarkSchema(AttrsSchema):
    class Meta:
        target = Watermark
        register_as_scheme = True


@attr.s(auto_attribs=True, kw_only=True)
class Stat:
    stat_type: str
    stat_val: Optional[str] = None
    start_epoch: Optional[int] = None
    end_epoch: Optional[int] = None


class StatSchema(AttrsSchema):
    class Meta:
        target = Stat
        register_as_scheme = True


@attr.s(auto_attribs=True, kw_only=True)
class Column:
    name: str
    key: Optional[str] = None
    description: Optional[str] = None
    col_type: str
    sort_order: int
    stats: List[Stat] = []
    badges: Optional[List[Badge]] = []


class ColumnSchema(AttrsSchema):
    class Meta:
        target = Column
        register_as_scheme = True


@attr.s(auto_attribs=True, kw_only=True)
class Application:
    application_url: Optional[str] = None
    description: Optional[str] = None
    id: str
    name: Optional[str] = None
    kind: Optional[str] = None


class ApplicationSchema(AttrsSchema):
    class Meta:
        target = Application
        register_as_scheme = True


@attr.s(auto_attribs=True, kw_only=True)
class Source:
    source_type: str
    source: str


class SourceSchema(AttrsSchema):
    class Meta:
        target = Source
        register_as_scheme = True


@attr.s(auto_attribs=True, kw_only=True)
class ResourceReport:
    name: str
    url: str


class ResourceReportSchema(AttrsSchema):
    class Meta:
        target = ResourceReport
        register_as_scheme = True


# this is a temporary hack to satisfy mypy. Once https://github.com/python/mypy/issues/6136 is resolved, use
# `attr.converters.default_if_none(default=False)`
def default_if_none(arg: Optional[bool]) -> bool:
    return arg or False


@attr.s(auto_attribs=True, kw_only=True)
class ProgrammaticDescription:
    source: str
    text: str


class ProgrammaticDescriptionSchema(AttrsSchema):
    class Meta:
        target = ProgrammaticDescription
        register_as_scheme = True


@attr.s(auto_attribs=True, kw_only=True)
class Table:
    database: str
    cluster: str
    schema: str
    name: str
    key: Optional[str] = None
    tags: List[Tag] = []
    badges: List[Badge] = []
    table_readers: List[Reader] = []
    description: Optional[str] = None
    columns: List[Column]
    owners: List[User] = []
    watermarks: List[Watermark] = []
    table_writer: Optional[Application] = None
    resource_reports: Optional[List[ResourceReport]] = None
    last_updated_timestamp: Optional[int] = None
    source: Optional[Source] = None
    is_view: Optional[bool] = attr.ib(default=None, converter=default_if_none)
    programmatic_descriptions: List[ProgrammaticDescription] = []
    integration_key: Optional[str] = None


class TableSchema(AttrsSchema):
    class Meta:
        target = Table
        register_as_scheme = True


@attr.s(auto_attribs=True, kw_only=True)
class TableSummary:
    database: str = attr.ib()
    cluster: str = attr.ib()
    schema: str = attr.ib()
    name: str = attr.ib()
    description: Optional[str] = attr.ib(default=None)
    schema_description: Optional[str] = attr.ib(default=None)
    last_updated_timestamp: Optional[int] = attr.ib(default=None)


class TableSummarySchema(AttrsSchema):
    class Meta:
        target = TableSummary
        register_as_scheme = True



from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_OAEP, AES
from Crypto.Random import get_random_bytes
import os
import base64


def check_key_path(key_path: str):
    if not os.path.exists(key_path) or not os.path.isfile(key_path):
        raise Exception(f'Key does not exist at {key_path}')


def encrypt(key_path: str, content: str):
    check_key_path(key_path)
    key = RSA.import_key(open(key_path, 'r').read())
    return base64.b64encode(PKCS1_OAEP.new(key).encrypt(content.encode('utf-8'))).decode('utf-8')


def decrypt(key_path: str, content: str):
    check_key_path(key_path)
    pkey = RSA.import_key(open(key_path, 'r').read())
    return PKCS1_OAEP.new(pkey).decrypt(base64.b64decode(content.encode('utf-8'))).decode('utf-8')


def big_encrypt(key_path: str, content: str) -> (str, str):
    check_key_path(key_path)
    aes_key = get_random_bytes(16)
    rsa_key = RSA.import_key(open(key_path, 'r').read())

    # Encrypt the data
    cipher = AES.new(aes_key, AES.MODE_EAX)
    ciphertext = base64.b64encode(cipher.encrypt(content.encode('utf-8'))).decode('utf-8')

    # Encrypt the AES key
    enc_key = base64.b64encode(PKCS1_OAEP.new(rsa_key).encrypt(aes_key)).decode('utf-8')

    # Return both
    # Note that a comma ',' is NOT a valid base64 character, so it is a safe deliminator
    return ciphertext, f'{enc_key},{base64.b64encode(cipher.nonce).decode("utf-8")}'


def big_decrypt(key_path: str, aes_string: str, content: str):
    check_key_path(key_path)
    rsa_key = RSA.import_key(open(key_path, 'r').read())
    enc_key, nonce = aes_string.split(',')

    # Decrypt the aes key
    aes_key = PKCS1_OAEP.new(rsa_key).decrypt(base64.b64decode(enc_key.encode('utf-8')))

    # Decrypt the data
    cipher = AES.new(aes_key, AES.MODE_EAX, base64.b64decode(nonce.encode('utf-8')))
    ciphertext = cipher\
        .decrypt(base64.b64decode(content.encode('utf-8')))\
        .decode('utf-8')

    # return both
    return ciphertext




from typing import Any, Optional


class ActionLogParams(object):
    """
    Holds parameters for Action log
    """
    def __init__(
        self, *,
        command: str,
        start_epoch_ms: int,
        end_epoch_ms: Optional[int] = 0,
        user: str,
        host_name: str,
        pos_args_json: str,
        keyword_args_json: str,
        output: Any = None,
        error: Optional[Exception] = None
    ) -> None:
        self.command = command
        self.start_epoch_ms = start_epoch_ms
        self.end_epoch_ms = end_epoch_ms
        self.user = user
        self.host_name = host_name
        self.pos_args_json = pos_args_json
        self.keyword_args_json = keyword_args_json
        self.output = output
        self.error = error

    def __repr__(self) -> str:
        return 'ActionLogParams(command={!r}, start_epoch_ms={!r}, end_epoch_ms={!r}, user={!r}, ' \
               'host_name={!r}, pos_args_json={!r}, keyword_args_json={!r}, output={!r}, error={!r})'\
            .format(
                self.command,
                self.start_epoch_ms,
                self.end_epoch_ms,
                self.user,
                self.host_name,
                self.pos_args_json,
                self.keyword_args_json,
                self.output,
                self.error,
            )



from setuptools import find_packages, setup

setup(
    name='secoda-common',
    version='0.1.16',
    description='Common code library for Secoda',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/secoda/common',
    maintainer='Secoda Inc.',
    maintainer_email='andrew.mcewen96@gmail.com',
    packages=find_packages(exclude=['tests*']),
    dependency_links=[
        ('git+https://www.github.com/hilearn/marshmallow-annotations.git@a7a2dc96932430369bd'
         'ef36555082df990ed9bef#egg=marshmallow-annotations')
    ],
    install_requires=[
        # Packages in here should rarely be pinned. This is because these
        # packages (at the specified version) are required for project
        # consuming this library. By pinning to a specific version you are the
        # number of projects that can consume this or forcing them to
        # upgrade/downgrade any dependencies pinned here in their project.
        #
        # Generally packages listed here are pinned to a major version range.
        #
        # e.g.
        # Python FooBar package for foobaring
        # pyfoobar>=1.0, <2.0
        #
        # This will allow for any consuming projects to use this library as
        # long as they have a version of pyfoobar equal to or greater than 1.x
        # and less than 2.x installed.
        'flask>=1.0.2',
        'marshmallow>=2.15.3,<=3.6',
        'marshmallow-annotations',
        'pycryptodome==3.10.1'
    ],
    python_requires=">=3.6",
    package_data={'secoda_common': ['py.typed']},
)
